(***********************************************************************)
(*                                                                     *)
(*                          Sklml                                      *)
(*                                                                     *)
(*             Pierre Weis (Pierre.Weis@inria.fr)                      *)
(*             Francois Clement (Francois.Clement@inria.fr)            *)
(*             Quentin Carbonneaux (Quentin.Carbonneaux@inria.fr)      *)
(*                                                                     *)
(* This program is free software.                                      *)
(* (see the file LICENSE in the root directory of the distribution).   *)
(*                                                                     *)
(***********************************************************************)

(* $Id: stream_on_wire.ml,v 1.14 2010-11-19 14:42:15 qcarbonn Exp $ *)

type 'a wire_data =
   | Data of 'a
   | Begin_sub_stream
   | End_sub_stream
;;

type 'a cell =
   | Nil
   | Cons of 'a wire_data * 'a stream

and 'a stream = 'a cell Lazy.t
;;

let nil_cell = Nil
and cons_begin_cell s = Cons (Begin_sub_stream, s)
and cons_end_cell s = Cons (End_sub_stream, s)
and cons_data_cell (x, s) = Cons (Data x, s)
and cons_wire_data_cell (d, s) = Cons (d, s)
;;

let get_cell = Lazy.force;;

(* Stream construction functions. *)
let nil = lazy Nil;;
let cons (x, s) = lazy (cons_data_cell (x, s));;

let cons_sub_stream (sub, s) =

  let rec loop sub =
    match get_cell sub with
    | Nil -> cons_end_cell s
    | Cons (d, sub) -> cons_wire_data_cell (d, lazy (loop sub)) in

  cons_begin_cell (lazy (loop sub))
;;

let rec append s1 s2 =

  let in_append s1 =
    match get_cell s1 with
    | Nil -> get_cell s2
    | Cons (d, s1) -> cons_wire_data_cell (d, append s1 s2) in

  lazy (in_append s1)
;;
