CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C                  projet Estime, INRIA-Rocquencourt                  C
C                                                                     C
C  Copyright 2004-2011 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: write_memory_mapping.F,v 1.5 2011-03-10 10:41:56 fclement Exp $

#include "pio_constants.h"

      subroutine string_of_communication_type(sbr, tc, len, stc)
C     Return the string stc of length len associated to the
C     communication type tc.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer tc
      
C     Output arguments.
      character stc*MAX_STRING_LENGTH
      integer len
      
      if (tc .eq. PIO_PHRASE) then
         len = 6
         stc(1:len) = 'Phrase'
      else if (tc .eq. PIO_TASK) then
         len = 4
         stc(1:len) = 'Task'
      else if (tc .eq. PIO_RESULT) then
         len = 6
         stc(1:len) = 'Result'
      else if (tc .eq. PIO_ERROR) then
         len = 5
         stc(1:len) = 'Error'
      else if (tc .eq. PIO_SERVICE) then
         len = 7
         stc(1:len) = 'Service'
      else
         len = 26
         stc(1:len) = 'Unknown communication type'
      endif
      
      if (len .gt. MAX_STRING_LENGTH) then
         call error_length_too_long(sbr, len, MAX_STRING_LENGTH)
      endif
      
      return
      end
      
      
      subroutine write_communication_type(io)
C     Write the communication type (see 'io_type_constants.h').
      
      implicit none
      
C     Input argument.
      integer io
      
C     Local variables.
      character sbr*24, stc*MAX_STRING_LENGTH
      integer tc, len
      
C     External function.
      integer fetch_communication_type
      
      sbr = 'write_communication_type'
      
      tc = fetch_communication_type(sbr, io)
      call string_of_communication_type(sbr, tc, len, stc)
      
      write(PIO_REPORT, *)
     +     '  Communication_Type (Tc, slot ',
     +     SLOT_COMMUNICATION_TYPE, ') = ', tc,
     +     ' (ie ', stc(1:len), ')'
      
      return
      end
      
      
      subroutine write_communication_name_bounds(io)
C     Write the communication name bounds.
      
      implicit none
      
C     Input argument.
      integer io
      
C     Local variables.
      character sbr*31
      integer tc, bn, en
      
C     External functions.
      integer fetch_communication_type
      integer fetch_communication_name_begin
      integer fetch_communication_name_end
      
      sbr = 'write_communication_name_bounds'
      
      tc = fetch_communication_type(sbr, io)
      
      if ((tc .eq. PIO_TASK) .or. (tc .eq. PIO_SERVICE)) then
         
         bn = fetch_communication_name_begin(sbr, io)
         write(PIO_REPORT, *)
     +        '  Name_Begin (bn, slot ',
     +        SLOT_NAME_BEGIN, ') = ', bn
         
         en = fetch_communication_name_end(sbr, io)
         write(PIO_REPORT, *)
     +        '  Name_End (en, slot ',
     +        SLOT_NAME_END, ') = ', en
         
      endif
      
      return
      end
      
      
      subroutine string_of_scalar_simple_type(typ, stv, len)
C     String of a scalar simple value type.
      
      implicit none
      
C     Input argument.
      integer typ
      
C     Input/output arguments.
      character stv*MAX_STRING_LENGTH
      integer len
      
C     Local variables.
      character s*MAX_STRING_LENGTH
      integer l
      
C     External functions.
      logical is_scalar_simple_string_type
      logical is_scalar_simple_int_type
      logical is_scalar_simple_float_type
      
      if (is_scalar_simple_string_type(typ)) then
         l = 6
         s(1:l) = 'String'
      else if (is_scalar_simple_int_type(typ)) then
         l = 3
         s(1:l) = 'Int'
      else if (is_scalar_simple_float_type(typ)) then
         l = 5
         s(1:l) = 'Float'
      else
         l = 32
         s(1:l) = 'Unknown scalar simple value type'
      endif
      
      stv(len+1:len+l) = s(1:l)
      len = len + l
      
      return
      end
      
      
      subroutine string_of_scalar_type(typ, stv, len)
C     String of a scalar value type.
      
      implicit none
      
C     Input argument.
      integer typ
      
C     Input/output arguments.
      character stv*MAX_STRING_LENGTH
      integer len
      
C     Local variables.
      character s*MAX_STRING_LENGTH
      integer item_typ, l
      
C     External functions.
      logical is_scalar_simple_type
      logical is_scalar_couple_type
      logical is_scalar_triple_type
      
      item_typ = MAKE_SCALAR_SIMPLE_ITEM_TYPE(typ)
      call string_of_scalar_simple_type(item_typ, stv, len)
      
      if (is_scalar_simple_type(typ)) then
         l = 0
      else if (is_scalar_couple_type(typ)) then
         l = 7
         s(1:l) = ' Couple'
      else if (is_scalar_triple_type(typ)) then
         l = 7
         s(1:l) = ' Triple'
      else
         l = 25
         s(1:l) = 'Unknown scalar value type'
      endif
      
      stv(len+1:len+l) = s(1:l)
      len = len + l
      
      return
      end
      
      
      subroutine string_of_vector_type(typ, stv, len)
C     String of a vector value type.
      
      implicit none
      
C     Input argument.
      integer typ
      
C     Input/output arguments.
      character stv*MAX_STRING_LENGTH
      integer len
      
C     Local variables.
      integer item_typ, l
      
      item_typ = MAKE_VECTOR_ITEM_TYPE(typ)
      call string_of_scalar_type(item_typ, stv, len)
      
      l = 7
      stv(len+1:len+l) = ' Vector'
      len = len + l
      
      return
      end
      
      
      subroutine string_of_multi_d_type(typ, stv, len)
C     String of a multi-dimensional value type.
      
      implicit none
      
C     Input argument.
      integer typ
      
C     Input/output arguments.
      character stv*MAX_STRING_LENGTH
      integer len
      
C     Local variables.
      integer item_typ, l
      
      item_typ = MAKE_MULTI_D_ITEM_TYPE(typ)
      call string_of_scalar_type(item_typ, stv, len)
      
      l = 8
      stv(len+1:len+l) = ' Multi_D'
      len = len + l
      
      return
      end
      
      
      integer function write_value_type(io, i, vdai)
C     Write the value type.
      
      implicit none
      
C     Input arguments.
      integer io, i, vdai
      
C     Local variables.
      character sbr*16, stv*MAX_STRING_LENGTH
      integer tv, len
      
C     External functions.
      logical is_scalar_type, is_vector_type, is_multi_d_type
      integer fetch_value_type
      
      sbr = 'write_value_type'
      
      tv = fetch_value_type(sbr, io, i)
      
      len = 0
      if (is_scalar_type(tv)) then
         call string_of_scalar_type(tv, stv, len)
      else if (is_vector_type(tv)) then
         call string_of_vector_type(tv, stv, len)
      else if (is_multi_d_type(tv)) then
         call string_of_multi_d_type(tv, stv, len)
      else
         len = 18
         stv(1:len) = 'Unknown value type'
      endif
      
      if (len .le. MAX_STRING_LENGTH) then
         write(PIO_REPORT, *)
     +        '    Value_Type ',
     +        '(Tv, slot ', vdai + SLOT_VALUE_TYPE, ') = ', tv,
     +        ' (ie ', stv(1:len), ')'
      else
         call error_length_too_long(sbr, len, MAX_STRING_LENGTH)
      endif
      
      write_value_type = tv
      
      return
      end
      
      
      subroutine write_array_sizes(io, i, vdai, pv)
C     Write the sizes of the array value.
      
      implicit none
      
C     Input arguments.
      integer io, i, vdai, pv
      
C     Local variables.
      character sbr*17
      integer sizev(MAX_ARRAY_DIMENSION), j
      
      sbr = 'write_array_sizes'
      
      write(PIO_REPORT, *) '    Value_Sizes begins'
      
      call fetch_value_sizes(sbr, io, i, sizev)
      
      do j = 1, pv
         write(PIO_REPORT, *)
     +        '      Value_Sizes (sizev, slot ',
     +        vdai + SLOT_VALUE_SIZES(j), ') = ', sizev(j)
      enddo
      
      write(PIO_REPORT, *) '    Value_Sizes ends'
      
      return
      end
      
      
      subroutine write_multi_d_layout(io, i, vdai, pv)
C     Write the layout of the multi-dimensional value.
      
      implicit none
      
C     Input arguments.
      integer io, i, vdai, pv
      
C     Local variables.
      character sbr*20, slkv*MAX_STRING_LENGTH
      integer lkv, len, layoutv(MAX_ARRAY_DIMENSION), j
      
C     External function.
      integer fetch_value_layout_kind
      
      sbr = 'write_multi_d_layout'
      
      lkv = fetch_value_layout_kind(sbr, io, i)
      
      if (lkv .eq. C_LAYOUT_KIND) then
         len = 8
         slkv(1:len) = 'C layout'
      else if (lkv .eq. FORTRAN_LAYOUT_KIND) then
         len = 14
         slkv(1:len) = 'Fortran layout'
      else if (lkv .eq. PERMUTATION_LAYOUT_KIND) then
         len = 18
         slkv(1:len) = 'Permutation layout'
      else
         len = 19
         slkv(1:len) = 'Unknown layout kind'
      endif
      
      if (len .le. MAX_STRING_LENGTH) then
         write(PIO_REPORT, *)
     +        '    Value_Layout_Kind (lkv, slot ',
     +        vdai + SLOT_VALUE_LAYOUT_KIND(pv), ') = ', lkv,
     +        ' (ie ', slkv(1:len), ')'
      else
         call error_length_too_long(sbr, len, MAX_STRING_LENGTH)
      endif
      
      if (lkv .eq. PERMUTATION_LAYOUT_KIND) then
         
         write(PIO_REPORT, *) '    Permutation Value_Layout begins'
         
         call fetch_value_layout(sbr, io, i, layoutv)
         
         do j = 1, pv
            write(PIO_REPORT, *)
     +        '      Value_Layout (layoutv, slot ',
     +        vdai + SLOT_VALUE_LAYOUT(pv, j), ') = ', layoutv(j)
         enddo
         
         write(PIO_REPORT, *) '    Permutation Value_Layout ends'
         
      endif
      
      return
      end
      
      
      subroutine write_string_lengths(io, i, vdai)
C     Write the lengths of the string value.
      
      implicit none
      
C     Input arguments.
      integer io, i, vdai
      
C     Local variables.
      character sbr*20
      integer slot, nl, lenv(MAX_STRING_ITEMS), k
      
C     External function.
      integer fetch_slot_value_lengths_begin
      
      sbr = 'write_string_lengths'
      
      write(PIO_REPORT, *) '    Value_Lengths begins'
      
      slot = fetch_slot_value_lengths_begin(sbr, io, i)
      call fetch_value_lengths(sbr, io, i, nl, lenv)
      
      do k = 1, nl
         write(PIO_REPORT, *)
     +        '      Value_Lengths (lenv, slot ',
     +        vdai + slot, ') = ', lenv(k)
         slot = slot + 1
      enddo
      
      write(PIO_REPORT, *) '    Value_Lengths ends'
      
      return
      end
      
      
      subroutine write_value_descriptor(io, i, vdai)
C     Write the value descriptor.
      
      implicit none
      
C     Input arguments.
      integer io, i, vdai
      
C     Local variables.
      character sbr*22
      integer tv, pv, bv, ev
      
C     External functions.
      logical is_array_type, is_multi_d_type, is_string_in_type
      integer write_value_type
      integer fetch_value_dimension, fetch_value_begin, fetch_value_end
      
      sbr = 'write_value_descriptor'
      
      tv = write_value_type(io, i, vdai)
      
      pv = fetch_value_dimension(sbr, io, i)
      write(PIO_REPORT, *)
     +     '    Value_Dimension (pv, slot ',
     +     vdai + SLOT_VALUE_DIMENSION, ') = ', pv
      
      bv = fetch_value_begin(sbr, io, i)
      write(PIO_REPORT,  *)
     +     '    Value_Begin (bv, slot ',
     +     vdai + SLOT_VALUE_BEGIN, ') = ', bv
      
      ev = fetch_value_end(sbr, io, i)
      write(PIO_REPORT,  *)
     +     '    Value_End (ev, slot ',
     +     vdai + SLOT_VALUE_END, ') = ', ev
      
      if (is_array_type(tv)) then
         call write_array_sizes(io, i, vdai, pv)
      endif
      
      if (is_multi_d_type(tv)) then
         call write_multi_d_layout(io, i, vdai, pv)
      endif
      
      if (is_string_in_type(tv)) then
         call write_string_lengths(io, i, vdai)
      endif
      
      return
      end
      
      
      subroutine write_value_descriptors(io, nv)
C     Write the value descriptors.
      
      implicit none
      
C     Input arguments.
      integer io, nv
      
C     Local variables.
      character sbr*23
      integer i, vdai
      
C     External function.
      integer fetch_communication_value_descriptor_address
      
      sbr = 'write_value_descriptors'
      
      write(PIO_REPORT, *) '  Value descriptors begin'
      
      do i = 1, nv
         
         write(PIO_REPORT, *) ' '
         write(PIO_REPORT, *) '    Value number ', i
         vdai = fetch_communication_value_descriptor_address(sbr, io, i)
         write(PIO_REPORT, *)
     +        '    Value_Descriptor_Address (vda, slot ',
     +        SLOT_VALUE_DESCRIPTOR_ADDRESS(i), ') = ', vdai
         
         call write_value_descriptor(io, i, vdai)
         
      enddo
      
      write(PIO_REPORT, *) ' '
      write(PIO_REPORT, *) '  Value descriptors end'
      
      return
      end
      
      
      subroutine pio_write_memory_mapping(io)
C     Write the memory mapping to stderr.
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input argument.
      integer io
      
C     Local variables.
      character sbr*20
      integer nv, bc, ec
      
C     External functions.
      integer fetch_communication_values_number
      integer fetch_communication_begin, fetch_communication_end
      
      sbr = 'write_memory_mapping'
      
      write(PIO_REPORT, *) memory_name(io+2), ' memory mapping begins'
      write(PIO_REPORT, *) ' '
      
      call write_communication_type(io)

      nv = fetch_communication_values_number(sbr, io)
      write(PIO_REPORT, *)
     +     '  Values_Number (Nv, slot ',
     +     SLOT_VALUES_NUMBER, ') = ', nv
      
      call write_communication_name_bounds(io)
      
      bc = fetch_communication_begin(sbr, io)
      write(PIO_REPORT, *)
     +     '  Communication_Begin (bc, slot ',
     +     SLOT_COMMUNICATION_BEGIN, ') = ', bc
      
      ec = fetch_communication_end(sbr, io)
      write(PIO_REPORT, *)
     +     '  Communication_End (ec, slot ',
     +     SLOT_COMMUNICATION_END, ') = ', ec
      
      write(PIO_REPORT, *) ' '
      
      call write_value_descriptors(io, nv)
      
      write(PIO_REPORT, *) ' '
      write(PIO_REPORT, *) memory_name(io+2), ' memory mapping ends.'
      
      return
      end
